/* ========================================
 *  Golem - Golem.h
 *  Copyright (c) 2016 airwindows, All rights reserved
 * ======================================== */

#ifndef __Golem_H
#include "Golem.h"
#endif

AudioEffect* createEffectInstance(audioMasterCallback audioMaster) {return new Golem(audioMaster);}

Golem::Golem(audioMasterCallback audioMaster) :
    AudioEffectX(audioMaster, kNumPrograms, kNumParameters)
{
    A = 0.5;
    B = 0.5;
    C = 0.0;
    for(count = 0; count < 4098; count++) {p[count] = 0.0;}
    count = 0;
    fpNShapeL = 0.0;
    fpNShapeR = 0.0;
    flip = true;
    //this is reset: values being initialized only once. Startup values, whatever they are.

    _canDo.insert("plugAsChannelInsert"); // plug-in can be used as a channel insert effect.
    _canDo.insert("plugAsSend"); // plug-in can be used as a send effect.
    _canDo.insert("x2in2out");
    setNumInputs(kNumInputs);
    setNumOutputs(kNumOutputs);
    setUniqueID(kUniqueId);
    canProcessReplacing();     // supports output replacing
    canDoubleReplacing();      // supports double precision processing
    programsAreChunks(true);
    vst_strncpy (_programName, "Default", kVstMaxProgNameLen); // default program name
}

Golem::~Golem() {}
VstInt32 Golem::getVendorVersion () {return 1000;}
void Golem::setProgramName(char *name) {vst_strncpy (_programName, name, kVstMaxProgNameLen);}
void Golem::getProgramName(char *name) {vst_strncpy (name, _programName, kVstMaxProgNameLen);}
//airwindows likes to ignore this stuff. Make your own programs, and make a different plugin rather than
//trying to do versioning and preventing people from using older versions. Maybe they like the old one!

static float pinParameter(float data)
{
    if (data < 0.0f) return 0.0f;
    if (data > 1.0f) return 1.0f;
    return data;
}

VstInt32 Golem::getChunk (void** data, bool isPreset)
{
    float *chunkData = (float *)calloc(kNumParameters, sizeof(float));
    chunkData[0] = A;
    chunkData[1] = B;
    chunkData[2] = C;
    /* Note: The way this is set up, it will break if you manage to save settings on an Intel
     machine and load them on a PPC Mac. However, it's fine if you stick to the machine you
     started with. */

    *data = chunkData;
    return kNumParameters * sizeof(float);
}

VstInt32 Golem::setChunk (void* data, VstInt32 byteSize, bool isPreset)
{
    float *chunkData = (float *)data;
    A = pinParameter(chunkData[0]);
    B = pinParameter(chunkData[1]);
    C = pinParameter(chunkData[2]);
    /* We're ignoring byteSize as we found it to be a filthy liar */

    /* calculate any other fields you need here - you could copy in
     code from setParameter() here. */
    return 0;
}

void Golem::setParameter(VstInt32 index, float value) {
    switch (index) {
        case kParamA: A = value; break;
        case kParamB: B = value; break;
        case kParamC: C = value; break;
        default: throw; // unknown parameter, shouldn't happen!
    }
}

float Golem::getParameter(VstInt32 index) {
    switch (index) {
        case kParamA: return A; break;
        case kParamB: return B; break;
        case kParamC: return C; break;
        default: break; // unknown parameter, shouldn't happen!
    } return 0.0; //we only need to update the relevant name, this is simple to manage
}

void Golem::getParameterName(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: vst_strncpy (text, "Balance", kVstMaxParamStrLen); break;
        case kParamB: vst_strncpy (text, "Offset", kVstMaxParamStrLen); break;
        case kParamC: vst_strncpy (text, "Phase", kVstMaxParamStrLen); break;
        default: break; // unknown parameter, shouldn't happen!
    } //this is our labels for displaying in the VST host
}

void Golem::getParameterDisplay(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: float2string ((A*2.0)-1.0, text, kVstMaxParamStrLen); break;
        case kParamB: float2string ((B*2.0)-1.0, text, kVstMaxParamStrLen); break;
        case kParamC: switch((VstInt32)( C * 5.999 )) //0 to almost edge of # of params
        {
            case 0: vst_strncpy (text, "In/Mic", kVstMaxParamStrLen); break;
            case 1: vst_strncpy (text, "Out/Mic", kVstMaxParamStrLen); break;
            case 2: vst_strncpy (text, "In/DI", kVstMaxParamStrLen); break;
            case 3: vst_strncpy (text, "Out/DI", kVstMaxParamStrLen); break;
            case 4: vst_strncpy (text, "DefTime", kVstMaxParamStrLen); break;
            case 5: vst_strncpy (text, "DefGain", kVstMaxParamStrLen); break;
         default: break; // unknown parameter, shouldn't happen!
        } break;

        default: break; // unknown parameter, shouldn't happen!
    } //this displays the values and handles 'popups' where it's discrete choices
}

void Golem::getParameterLabel(VstInt32 index, char *text) {
    switch (index) {
        case kParamA: vst_strncpy (text, "", kVstMaxParamStrLen); break;
        case kParamB: vst_strncpy (text, "", kVstMaxParamStrLen); break;
        case kParamC: vst_strncpy (text, "", kVstMaxParamStrLen); break;
        default: break; // unknown parameter, shouldn't happen!
    }
}

VstInt32 Golem::canDo(char *text)
{ return (_canDo.find(text) == _canDo.end()) ? -1: 1; } // 1 = yes, -1 = no, 0 = don't know

bool Golem::getEffectName(char* name) {
    vst_strncpy(name, "Golem", kVstMaxProductStrLen); return true;
}

VstPlugCategory Golem::getPlugCategory() {return kPlugCategEffect;}

bool Golem::getProductString(char* text) {
    vst_strncpy (text, "airwindows Golem", kVstMaxProductStrLen); return true;
}

bool Golem::getVendorString(char* text) {
    vst_strncpy (text, "airwindows", kVstMaxVendorStrLen); return true;
}
