/*
    ,--.                     ,--.     ,--.  ,--.
  ,-'  '-.,--.--.,--,--.,---.|  |,-.,-'  '-.`--' ,---. ,--,--,      Copyright 2018
  '-.  .-'|  .--' ,-.  | .--'|     /'-.  .-',--.| .-. ||      \   Tracktion Software
    |  |  |  |  \ '-'  \ `--.|  \  \  |  |  |  |' '-' '|  ||  |       Corporation
    `---' `--'   `--`--'`---'`--'`--' `---' `--' `---' `--''--'    www.tracktion.com

    Tracktion Engine uses a GPL/commercial licence - see LICENCE.md for details.
*/

#pragma once

//==============================================================================
/*
    These hash functions are from boost and so the boost license agreement and
    attribution are included here:
*/
//==============================================================================
/*
    Boost Software License - Version 1.0 - August 17th, 2003

    Permission is hereby granted, free of charge, to any person or organization
    obtaining a copy of the software and accompanying documentation covered by
    this license (the "Software") to use, reproduce, display, distribute,
    execute, and transmit the Software, and to prepare derivative works of the
    Software, and to permit third-parties to whom the Software is furnished to
    do so, all subject to the following:

    The copyright notices in the Software and this entire statement, including
    the above license grant, this restriction and the following disclaimer,
    must be included in all copies of the Software, in whole or in part, and
    all derivative works of the Software, unless such copies or derivative
    works are solely in the form of machine-executable object code generated by
    a source language processor.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
    SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
    FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
    ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*/
//==============================================================================

namespace tracktion_graph
{

//==============================================================================
//==============================================================================
template<typename T>
void hash_combine (size_t& seed, const T& v)
{
    seed ^= std::hash<T>()(v) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
}

template<typename It>
std::size_t hash_range (It first, It last)
{
    size_t seed = 0;

    for (; first != last; ++first)
        hash_combine (seed, *first);

    return seed;
}

template<typename It>
void hash_range (std::size_t& seed, It first, It last)
{
    for (; first != last; ++first)
        hash_combine (seed, *first);
}

/** Hashes a type with a given seed and returns the new hash value. */
template<typename T>
size_t hash (size_t seed, const T& v)
{
    hash_combine (seed, v);
    return seed;
}

//==============================================================================
//==============================================================================
/** Converts an integer sample number to a time in seconds. */
template<typename IntType>
constexpr double sampleToTime (IntType samplePosition, double sampleRate)
{
    return samplePosition / sampleRate;
}

/** Converts a time in seconds to a sample number. */
constexpr int64_t timeToSample (double timeInSeconds, double sampleRate)
{
    return static_cast<int64_t> ((timeInSeconds * sampleRate) + 0.5);
}

/** Converts an integer sample range to a time range in seconds. */
template<typename IntType>
constexpr juce::Range<double> sampleToTime (juce::Range<IntType> sampleRange, double sampleRate)
{
    return { sampleToTime (sampleRange.getStart(), sampleRate),
             sampleToTime (sampleRange.getEnd(), sampleRate) };
}

/** Converts a time range in seconds to a range of sample numbers. */
constexpr juce::Range<int64_t> timeToSample (juce::Range<double> timeInSeconds, double sampleRate)
{
    return { timeToSample (timeInSeconds.getStart(), sampleRate),
             timeToSample (timeInSeconds.getEnd(), sampleRate) };
}

/** Converts a time range in seconds to a range of sample numbers. */
template<typename RangeType>
constexpr juce::Range<int64_t> timeToSample (RangeType timeInSeconds, double sampleRate)
{
    return { timeToSample (timeInSeconds.getStart(), sampleRate),
             timeToSample (timeInSeconds.getEnd(), sampleRate) };
}

}
