/*
  ==============================================================================

   This file is part of the JUCE library.
   Copyright (c) 2020 - Raw Material Software Limited

   JUCE is an open source library subject to commercial or open-source
   licensing.

   The code included in this file is provided under the terms of the ISC license
   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
   To use, copy, modify, and/or distribute this software for any purpose with or
   without fee is hereby granted provided that the above copyright notice and
   this permission notice appear in all copies.

   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
   DISCLAIMED.

  ==============================================================================
*/

namespace juce
{

struct PhysicalTopologySource::DetectorHolder  : private Timer
{
    DetectorHolder (PhysicalTopologySource& pts)
        : topologySource (pts),
          detector (Detector::getDefaultDetector())
    {
        startTimerHz (30);
    }

    DetectorHolder (PhysicalTopologySource& pts, DeviceDetector& dd)
        : topologySource (pts),
          detector (new Detector (dd))
    {
        startTimerHz (30);
    }

    void timerCallback() override
    {
        if (! topologySource.hasOwnServiceTimer())
            handleTimerTick();
    }

    void handleTimerTick()
    {
        auto blocks = detector->currentTopology.blocks;

        if (blocks.size() == 0)
            return;

        if (nextIndexToTick >= blocks.size())
            nextIndexToTick = 0;

        if (auto* bi = BlockImplementation<Detector>::getFrom (*blocks [nextIndexToTick]))
            bi->handleTimerTick();

        nextIndexToTick = (nextIndexToTick + 1) % blocks.size();
    }

    PhysicalTopologySource& topologySource;
    Detector::Ptr detector;

    int nextIndexToTick {0};
};

} // namespace juce
